unit URouteCalculator;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.TMSFNCTypes, FMX.TMSFNCUtils, FMX.TMSFNCGraphics, FMX.TMSFNCGraphicsTypes,
  FMX.TMSFNCCustomComponent, FMX.TMSFNCRouteCalculator,
  FMX.Controls.Presentation, FMX.StdCtrls, FMX.TMSFNCCustomControl,
  FMX.TMSFNCWebBrowser, FMX.TMSFNCMaps, FMX.Edit, FMX.ListBox, FMX.TMSFNCGeocoding,
  FMX.TMSFNCCloudBase, FMX.TMSFNCMapsCommonTypes;

type
  TForm4 = class(TForm)
    TMSFNCMaps1: TTMSFNCMaps;
    Panel1: TPanel;
    TMSFNCRouteCalculator1: TTMSFNCRouteCalculator;
    Label1: TLabel;
    cbMapService: TComboBox;
    Label2: TLabel;
    cbDirectionsService: TComboBox;
    edMapAPIKey: TEdit;
    edDirectionsAPIKey: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    Panel2: TPanel;
    Label7: TLabel;
    btUndo: TButton;
    btRedo: TButton;
    btClear: TButton;
    Label8: TLabel;
    edLocation: TEdit;
    btGoToLocation: TButton;
    btDelete: TButton;
    Label6: TLabel;
    Label5: TLabel;
    btExport: TButton;
    btImport: TButton;
    SaveDialog1: TSaveDialog;
    OpenDialog1: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure cbMapServiceChange(Sender: TObject);
    procedure btUndoClick(Sender: TObject);
    procedure btRedoClick(Sender: TObject);
    procedure btClearClick(Sender: TObject);
    procedure btGoToLocationClick(Sender: TObject);
    procedure TMSFNCRouteCalculator1GetGeocoding(Sender: TObject;
      const ARequest: TTMSFNCGeocodingRequest;
      const ARequestResult: TTMSFNCCloudBaseRequestResult);
    procedure btDeleteClick(Sender: TObject);
    procedure btExportClick(Sender: TObject);
    procedure btImportClick(Sender: TObject);
    procedure TMSFNCMaps1RouteCalculatorBeforeDeleteMarker(Sender: TObject;
      var ACanDelete: Boolean; AMarker: TTMSFNCMapsMarker;
      ASegment: TTMSFNCRouteCalculatorSegment);
    procedure TMSFNCMaps1RouteCalculatorBeforeDeletePolyline(Sender: TObject;
      var ACanDelete: Boolean; APolyline: TTMSFNCMapsPolyline;
      ASegment: TTMSFNCRouteCalculatorSegment);
  private
    { Private declarations }
  public
    { Public declarations }
    mapkey, directionskey, googlekey, herekey: string;
    procedure SetAPIKey;
  end;

var
  Form4: TForm4;

implementation

{$R *.fmx}

procedure TForm4.btClearClick(Sender: TObject);
begin
  TMSFNCMaps1.Clear;
  TMSFNCRouteCalculator1.Clear;
end;

procedure TForm4.btRedoClick(Sender: TObject);
begin
  TMSFNCRouteCalculator1.HistoryManager.Redo;
  TMSFNCMaps1.Clear;
  TMSFNCMaps1.RouteCalculatorPlotRoutes;
end;

procedure TForm4.btUndoClick(Sender: TObject);
begin
  TMSFNCRouteCalculator1.HistoryManager.Undo;
  TMSFNCMaps1.Clear;
  TMSFNCMaps1.RouteCalculatorPlotRoutes;
end;

procedure TForm4.btDeleteClick(Sender: TObject);
begin
  if Assigned(TMSFNCMaps1.RouteCalculatorSelectedPolyline) or Assigned(TMSFNCMaps1.RouteCalculatorSelectedMarker) then
  begin
    if TTMSFNCUtils.Message('Are you sure you want to delete the selected part?', TMsgDlgType.mtConfirmation, [TMsgDlgBtn.mbYes, TMsgDlgBtn.mbNo], 0) = 6 then
    begin
      TMSFNCMaps1.RouteCalculatorDeleteSelectedPolyline;
      TMSFNCMaps1.RouteCalculatorDeleteSelectedMarker;
    end
  end
  else
  begin
    ShowMessage('No route waypoint or segment selected');
  end;
end;

procedure TForm4.btExportClick(Sender: TObject);
var
  md: TTMSFNCMapsGPXMetaData;
begin
  if TMSFNCRouteCalculator1.HasRoutes then
  begin
    TMSFNCRouteCalculator1.Routes[0].RouteName := 'TMSFNCRouteCalculator';
    SaveDialog1.FileName := TMSFNCRouteCalculator1.Routes[0].RouteName + '.gpx';
    if SaveDialog1.Execute then
    begin
      md.TrackName := TMSFNCRouteCalculator1.Routes[0].RouteName;
      TMSFNCRouteCalculator1.SaveToGPXFile(TMSFNCRouteCalculator1.Routes[0], SaveDialog1.FileName, md)
    end;
  end
  else
  begin
    ShowMessage('Nothing to export');
  end;
end;

procedure TForm4.btGoToLocationClick(Sender: TObject);
begin
  TMSFNCRouteCalculator1.GetGeocoding(edLocation.Text);
end;

procedure TForm4.btImportClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    TMSFNCRouteCalculator1.LoadGPXFromFile(OpenDialog1.FileName);
    TMSFNCMaps1.RouteCalculatorPlotRoutes;
    if TMSFNCRouteCalculator1.HasRoutes then
      TMSFNCMaps1.ZoomToBounds(TMSFNCRouteCalculator1.Routes[0].Polyline);
  end;
end;

procedure TForm4.cbMapServiceChange(Sender: TObject);
begin
  SetAPIKey;
end;

procedure TForm4.FormCreate(Sender: TObject);
begin
  TMSFNCRouteCalculator1.Active := True;
  TMSFNCRouteCalculator1.Options.HistoryEnabled := True;
  TMSFNCMaps1.RouteCalculator := TMSFNCRouteCalculator1;
  SetAPIKey;
end;

procedure TForm4.SetAPIKey;
begin
  if edMapAPIKey.Text <> '' then
    mapkey := edMapAPIKey.Text;

  if edDirectionsAPIKey.Text <> '' then
    directionskey := edDirectionsAPIKey.Text;

  if cbMapService.ItemIndex = 0 then
  begin
    TMSFNCMaps1.Service := msOpenLayers;
    TMSFNCMaps1.APIKey := '';
  end
  else if cbMapService.ItemIndex = 1 then
  begin
    TMSFNCMaps1.Service := msGoogleMaps;
    TMSFNCMaps1.APIKey := mapkey;
  end
  else if cbMapService.ItemIndex = 2 then
  begin
    TMSFNCMaps1.Service := msHere;
    TMSFNCMaps1.APIKey := mapkey;
  end;

  if cbDirectionsService.ItemIndex = 0 then
  begin
    TMSFNCRouteCalculator1.Service := csGoogle;
    TMSFNCRouteCalculator1.APIKey := directionskey;
  end
  else if cbDirectionsService.ItemIndex = 1 then
  begin
    TMSFNCRouteCalculator1.Service := csHere;
    TMSFNCRouteCalculator1.APIKey := directionskey;
  end;
end;

procedure TForm4.TMSFNCMaps1RouteCalculatorBeforeDeleteMarker(Sender: TObject;
  var ACanDelete: Boolean; AMarker: TTMSFNCMapsMarker;
  ASegment: TTMSFNCRouteCalculatorSegment);
begin
  ACanDelete := (TTMSFNCUtils.Message('Are you sure you want to delete the selected waypoint?', TMsgDlgType.mtConfirmation, [TMsgDlgBtn.mbYes, TMsgDlgBtn.mbNo], 0) = 6)
end;

procedure TForm4.TMSFNCMaps1RouteCalculatorBeforeDeletePolyline(Sender: TObject;
  var ACanDelete: Boolean; APolyline: TTMSFNCMapsPolyline;
  ASegment: TTMSFNCRouteCalculatorSegment);
begin
  ACanDelete := (TTMSFNCUtils.Message('Are you sure you want to delete the selected segment?', TMsgDlgType.mtConfirmation, [TMsgDlgBtn.mbYes, TMsgDlgBtn.mbNo], 0) = 6)
end;

procedure TForm4.TMSFNCRouteCalculator1GetGeocoding(Sender: TObject;
  const ARequest: TTMSFNCGeocodingRequest;
  const ARequestResult: TTMSFNCCloudBaseRequestResult);
begin
  if ARequestResult.Success then
    TMSFNCMaps1.SetCenterCoordinate(ARequest.Items[0].Coordinate.ToRec);
end;

end.
